/// <reference types="isaac-typescript-definitions" />
import type { RockAltType } from "../enums/RockAltType";
/**
 * Helper function to get the alternate rock type (i.e. urn, mushroom, etc.) that the current room
 * will have.
 *
 * The rock type is based on the backdrop of the room.
 *
 * For example, if you change the backdrop of the starting room of the run to `BackdropType.CAVES`,
 * and then spawn `GridEntityType.ROCK_ALT`, it will be a mushroom instead of an urn. Additionally,
 * if it is destroyed, it will generate mushroom-appropriate rewards.
 *
 * On the other hand, if an urn is spawned first before the backdrop is changed to
 * `BackdropType.CAVES`, the graphic of the urn will not switch to a mushroom. However, when
 * destroyed, the urn will still generate mushroom-appropriate rewards.
 */
export declare function getRockAltType(): RockAltType;
/**
 * Helper function to remove all coins, trinkets, and so on that spawned from breaking an urn.
 *
 * The rewards are based on the ones from the wiki:
 * https://bindingofisaacrebirth.fandom.com/wiki/Rocks#Urns
 */
export declare function removeUrnRewards(gridEntity: GridEntity): void;
//# sourceMappingURL=rockAlt.d.ts.map